using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using LibJasinski.Network;

namespace DesktopText
{
	public partial class MainForm : Form
	{
		private string _oldProxyServer;
		private ProxyState _oldProxyState;
		private bool _textShown, _tempShown;

		//= Public Constructor and Event Handlers =================================================
		public MainForm()
		{
			InitializeComponent();
			//EnforcePositionSettings();
			RefreshText();
			_oldProxyServer = VerseGetter2.Default.ProxyServer;
			_oldProxyState = VerseGetter2.Default.ProxyState;
			_textShown = false;
			_tempShown = false;
		}
		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			switch (e.CloseReason)
			{
				case CloseReason.UserClosing:
					Size oldSize = this.Size;
					this.FormBorderStyle = FormBorderStyle.None;
					ResetPosition(oldSize);
					e.Cancel = true;
					break;
			}
		}
		private void uxChangeTextMenuItem_Click(object sender, EventArgs e)
		{
			//GetTextForm f = new GetTextForm();
			//if (f.GetText())
			//    this.RefreshText();
		}
		private void uxExitMenuItem_Click(object sender, EventArgs e)
		{
			Cfg.Def.WindowTop = this.Top;
			Cfg.Def.WindowLeft = this.Left;
			Cfg.Def.Save();
			Application.Exit();
		}
		private void uxNotifyIcon_MouseClick(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				Size oldSize = this.Size;

				if (FormBorderStyle == FormBorderStyle.FixedToolWindow)
					FormBorderStyle = FormBorderStyle.None;
				else
					FormBorderStyle = FormBorderStyle.FixedToolWindow;

				ResetPosition(oldSize);
			}
		}
		private void uxTimer_Tick(object sender, EventArgs e)
		{
			uxTimer.Enabled = false;
			//DateTime last = Cfg.Def.LastUpdate;
			int n = Cfg.Def.RefreshFrequency;

			//- Scripture Verse -------------------------------------------------------------------
			TimeSpan delta = DateTime.Now.Subtract(Cfg.Def.LastUpdate);
			if (delta.TotalMinutes >= n && VerseGetter2.Default.Get())
			{
				uxNotifyIcon.Text = "Desktop Text\nProxy Server: " + VerseGetter2.Default.ProxyServer;
				string oldVerse = Cfg.Def.Verse;
				string oldSource = Cfg.Def.Source;
				string newVerse = VerseGetter2.Default.Verse;
				string newSource = VerseGetter2.Default.Reference;

				if (oldVerse != newVerse | oldSource != newSource & !_textShown)
				{
					Cfg.Def.Verse = newVerse;
					Cfg.Def.Source = newSource;
					Cfg.Def.Save();
					this.RefreshText();
				}
			}
			else if (!_textShown)
				RefreshText();

			//- Weather ---------------------------------------------------------------------------
			delta = DateTime.Now.Subtract(Cfg.Def.WeatherTimeStamp);
			if (delta.TotalMinutes >= n && WeatherGetter.Default.Get())
			{
				string oldTemp = Cfg.Def.Temperature;
				string oldCond = Cfg.Def.Conditions;
				string newTemp = WeatherGetter.Default.Temperature;
				string newCond = WeatherGetter.Default.Conditions;

				if (oldTemp != newTemp | oldCond != newCond | !_tempShown)
				{
					Cfg.Def.Temperature = newTemp;
					Cfg.Def.Conditions = newCond;
					Cfg.Def.WeatherTimeStamp = WeatherGetter.Default.TimeStamp;
					Cfg.Def.Save();
					RefreshWeather();
				}
			}
			else if (!_tempShown)
				RefreshWeather();

			HandleProxyStateChange();
			uxTimer.Enabled = true;
		}

		//= Private Methods =======================================================================
		private void AutoSizeLabel(Label target, string s)
		{
			target.Text = s;
			Graphics g = target.CreateGraphics();
			SizeF sf = g.MeasureString(target.Text, target.Font);
			target.Size = new Size(
				(int)sf.Width + 4, 
				(int)sf.Height + 4
				);
			g.Dispose();
		}
		private void EnforcePositionSettings()
		{
			Rectangle r = Screen.PrimaryScreen.WorkingArea;
			int top = Cfg.Def.WindowTop;
			int left = Cfg.Def.WindowLeft;

			if (top < r.Top | top > r.Bottom)
				top = (r.Height - this.Height) / 2;

			if (left < r.Left | left > r.Right)
				left = (r.Width - this.Width) / 2;

			this.Top = top;
			this.Left = left;
		}
		private void HandleProxyStateChange()
		{
			//string newProxy = VerseGetter2.Default.ProxyServer;

			//if (_oldProxyServer != newProxy | _oldProxyState != VerseGetter2.Default.ProxyState)
			//{
			//    string msg = "Proxy server was lost";
			//    ToolTipIcon icon = ToolTipIcon.Warning;

			//    if (newProxy != VerseGetter2.NotApplicable)
			//    {
			//        msg = "Proxy server was found: " + VerseGetter2.Default.ProxyServer;
			//        icon = ToolTipIcon.Info;
			//    }
			//    uxNotifyIcon.ShowBalloonTip(1000, "Proxy Server", msg, icon);
			//}
			//_oldProxyServer = newProxy;
			//_oldProxyState = VerseGetter2.Default.ProxyState;
		}
		private void RefreshText()
		{
			Cfg.Def.LastUpdate = DateTime.Now;
			Cfg.Def.Save();
			AutoSizeLabel(uxVerseLabel, Cfg.Def.Verse);
			AutoSizeLabel(uxSourceLabel, Cfg.Def.Source);
			uxSourceLabel.Top = uxVerseLabel.Bottom + 3;
			uxSourceLabel.Left = uxVerseLabel.Right - uxSourceLabel.Width;
			this.Width = uxVerseLabel.Left * 2 + uxVerseLabel.Width;
			this.Height = uxSourceLabel.Bottom + uxVerseLabel.Top;
			// Anchor form to lower right corner of screen
			this.Left  = Screen.PrimaryScreen.WorkingArea.Width - this.Width;
			this.Top = Screen.PrimaryScreen.WorkingArea.Height - this.Height;
			_textShown = true;
		}
		private void RefreshWeather()
		{
			TextIcon i = new TextIcon();
			i.Text = Cfg.Def.Temperature;
			string s = Cfg.Def.Conditions.ToUpper();

			if (s.Contains("RAIN"))
				i.ForeColor = Color.Red;
			else if (s.Contains("CLOUD"))
				i.ForeColor = Color.FromArgb(128,128,128);
			else
				i.ForeColor = Color.Black;


			uxTempIcon.Icon = i.GetIcon();
			uxTempIcon.Text = Cfg.Def.Conditions + "\n" 
				+ Cfg.Def.WeatherTimeStamp.ToString("MM/dd/yy h:mm tt");
			_tempShown = true;
		}
		private void ResetPosition(Size oldSize)
		{
			int borderThickness = (this.Width - oldSize.Width) / 2;
			this.Top -= this.Height - oldSize.Height - borderThickness;
			this.Left -= borderThickness;
		}
	}
}